@@ -175,6 +175,7 @@ _metagen_general_status_get_fcn (const NMMetaEnvironment *environment,
                                  NMMetaAccessorGetType get_type,
                                  NMMetaAccessorGetFlags get_flags,
                                  NMMetaAccessorGetOutFlags *out_flags,
+                                 gboolean *out_is_default,
                                  gpointer *out_to_free)
 {
 	NmCli *nmc = target;
@@ -282,6 +283,7 @@ _metagen_general_permissions_get_fcn (const NMMetaEnvironment *environment,
                                       NMMetaAccessorGetType get_type,
                                       NMMetaAccessorGetFlags get_flags,
                                       NMMetaAccessorGetOutFlags *out_flags,
+                                      gboolean *out_is_default,
                                       gpointer *out_to_free)
 {
 	NMClientPermission perm = GPOINTER_TO_UINT (target);
@@ -330,6 +332,7 @@ _metagen_general_logging_get_fcn (const NMMetaEnvironment *environment,
                                   NMMetaAccessorGetType get_type,
                                   NMMetaAccessorGetFlags get_flags,
                                   NMMetaAccessorGetOutFlags *out_flags,
+                                  gboolean *out_is_default,
                                   gpointer *out_to_free)
 {
 	NmCli *nmc = environment_user_data;
@@ -743,7 +746,7 @@ do_general_logging (NmCli *nmc, int argc, char **argv)
 					                             "AGENTS", "SETTINGS", "SUSPEND", "CORE", "DEVICE", "OLPC",
 					                             "INFINIBAND", "FIREWALL", "ADSL", "BOND", "VLAN", "BRIDGE",
 					                             "DBUS_PROPS", "TEAM", "CONCHECK", "DCB", "DISPATCH", "AUDIT",
-					                             "SYSTEMD", "VPN_PLUGIN", "PROXY", NULL);
+					                             "SYSTEMD", "VPN_PLUGIN", "PROXY", "TC", NULL);
 				}
 				domains = *argv;
 			} else {
