@@ -21,15 +21,9 @@
 #define NMC_NMCLI_H
 
 #include "nm-secret-agent-old.h"
-
 #include "nm-meta-setting-desc.h"
 
-#if WITH_POLKIT_AGENT
-#include "nm-polkit-listener.h"
-#else
-/* polkit agent is not available; define fake NMPolkitListener */
-typedef gpointer NMPolkitListener;
-#endif
+struct _NMPolkitListener;
 
 typedef char *(*NmcCompEntryFunc) (const char *, int);
 
@@ -116,6 +110,7 @@ typedef struct _NmcConfig {
 	bool escape_values;                               /* Whether to escape ':' and '\' in terse tabular mode */
 	bool in_editor;                                   /* Whether running the editor - nmcli con edit' */
 	bool show_secrets;                                /* Whether to display secrets (both input and output): option '--show-secrets' */
+	bool overview;                                    /* Overview mode (hide default values) */
 } NmcConfig;
 
 typedef struct _NmcOutputData {
@@ -134,7 +129,7 @@ typedef struct _NmCli {
 
 	NMSecretAgentOld *secret_agent;                   /* Secret agent */
 	GHashTable *pwds_hash;                            /* Hash table with passwords in passwd-file */
-	NMPolkitListener *pk_listener ;                   /* polkit agent listener */
+	struct _NMPolkitListener *pk_listener;            /* polkit agent listener */
 
 	int should_wait;                                  /* Semaphore indicating whether nmcli should not end or not yet */
 	gboolean nowait_flag;                             /* '--nowait' option; used for passing to callbacks */
