@@ -19,10 +19,6 @@
 
 #include "nm-default.h"
 
-#include "NetworkManager.h"
-
-#include "nm-utils/nm-hash-utils.h"
-
 #include "nm-meta-setting-access.h"
 
 #include "nm-utils/nm-test-utils.h"
@@ -90,7 +86,7 @@ test_client_meta_check (void)
 				g_assert (pi->setting_info == info);
 				g_assert (pi->property_name);
 
-				g_assert (nm_g_hash_table_add (property_names, (gpointer) pi->property_name));
+				g_assert (g_hash_table_add (property_names, (gpointer) pi->property_name));
 
 				g_assert_cmpstr (pi->property_name, ==, pi->meta_type->get_name ((const NMMetaAbstractInfo *) pi, FALSE));
 				g_assert_cmpstr (pi->property_name, ==, pi->meta_type->get_name ((const NMMetaAbstractInfo *) pi, TRUE));
@@ -104,14 +100,14 @@ test_client_meta_check (void)
 
 		if (info->valid_parts) {
 			gsize i, l;
-			gs_unref_hashtable GHashTable *dup = g_hash_table_new (NULL, NULL);
+			gs_unref_hashtable GHashTable *dup = g_hash_table_new (nm_direct_hash, NULL);
 
 			l = NM_PTRARRAY_LEN (info->valid_parts);
 			g_assert (l >= 2);
 
 			for (i = 0; info->valid_parts[i]; i++) {
 				g_assert (info->valid_parts[i]->setting_info);
-				g_assert (nm_g_hash_table_add (dup, (gpointer) info->valid_parts[i]->setting_info));
+				g_assert (g_hash_table_add (dup, (gpointer) info->valid_parts[i]->setting_info));
 
 				if (i == 0) {
 					g_assert (info->valid_parts[i]->setting_info == &nm_meta_setting_infos_editor[NM_META_SETTING_TYPE_CONNECTION]);
