@@ -24,7 +24,7 @@
  * much of the low-level stuff for you.
  *
  * Compile with:
- *   gcc -Wall `pkg-config --libs --cflags glib-2.0 libnm` add-connection-libnm.c -o add-connection-libnm
+ *   gcc -Wall add-connection-libnm.c -o add-connection-libnm `pkg-config --libs --cflags libnm`
  */
 
 #include <glib.h>
@@ -37,7 +37,7 @@ added_cb (GObject *client,
 {
 	GMainLoop *loop = user_data;
 	NMRemoteConnection *remote;
-	GError *error;
+	GError *error = NULL;
 
 	/* NM responded to our request; either handle the resulting error or
 	 * print out the object path of the connection we just added.
@@ -105,11 +105,6 @@ main (int argc, char *argv[])
 	GMainLoop *loop;
 	GError *error = NULL;
 
-#if !GLIB_CHECK_VERSION (2, 35, 0)
-	/* Initialize GType system */
-	g_type_init ();
-#endif
-
 	loop = g_main_loop_new (NULL, FALSE);
 
 	/* Connect to NetworkManager */
