@@ -22,7 +22,7 @@
  * libnm.
  *
  * Compile with:
- *   gcc -Wall `pkg-config --libs --cflags glib-2.0 libnm` get-ap-info-libnm.c -o get-ap-info-libnm
+ *   gcc -Wall get-ap-info-libnm.c -o get-ap-info-libnm `pkg-config --libs --cflags libnm`
  */
 
 #include <glib.h>
@@ -79,7 +79,7 @@ show_access_point_info (NMAccessPoint *ap)
 {
 	guint32 flags, wpa_flags, rsn_flags, freq, bitrate;
 	guint8 strength;
-	GBytes *ssid; 
+	GBytes *ssid;
 	const char *hwaddr;
 	NM80211Mode mode;
 	char *freq_str, *ssid_str, *bitrate_str, *strength_str, *wpa_flags_str, *rsn_flags_str;
@@ -158,7 +158,7 @@ show_wifi_device_info (NMDevice *device)
 	const char *iface;
 	const char *driver;
 	guint32 speed;
-	GBytes *active_ssid; 
+	GBytes *active_ssid;
 	char *active_ssid_str = NULL;
 	int i;
 
@@ -202,11 +202,6 @@ main (int argc, char *argv[])
 	int i;
 	GError *error = NULL;
 
-#if !GLIB_CHECK_VERSION (2, 35, 0)
-	/* Initialize GType system */
-	g_type_init ();
-#endif
-
 	/* Get NMClient object */
 	client = nm_client_new (NULL, &error);
 	if (!client) {
