@@ -195,7 +195,7 @@ nm_keyfile_plugin_kf_get_keys (GKeyFile *kf,
 		alias = nm_keyfile_plugin_get_alias_for_setting_name (group);
 		if (alias) {
 			g_clear_error (&local);
-			keys = g_key_file_get_keys (kf, alias, out_length, &local);
+			keys = g_key_file_get_keys (kf, alias, out_length, error ? &local : NULL);
 		}
 	}
 	if (local)
@@ -442,7 +442,7 @@ _keyfile_key_encode (const char *name,
 		        && g_ascii_isxdigit (name[i + 2]))
 		    || (   ch == ' '
 		        && name[i + 1] == '\0'))
-			g_string_append_printf (str, "\\%2X", ch);
+			g_string_append_printf (str, "\\%02X", ch);
 		else
 			g_string_append_c (str, (char) ch);
 	}
