@@ -15,7 +15,7 @@
  * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301 USA.
  *
- * Copyright 2005 - 2014 Red Hat, Inc.
+ * Copyright 2005 - 2017 Red Hat, Inc.
  */
 
 #ifndef __NM_UTILS_H__
@@ -35,9 +35,12 @@
 
 #include "nm-core-enum-types.h"
 #include "nm-setting-wireless-security.h"
+#include "nm-setting-tc-config.h"
 
 G_BEGIN_DECLS
 
+typedef struct _NMVariantAttributeSpec NMVariantAttributeSpec;
+
 /* SSID helpers */
 gboolean    nm_utils_is_empty_ssid (const guint8 *ssid, gsize len);
 const char *nm_utils_escape_ssid   (const guint8 *ssid, gsize len);
@@ -215,6 +218,36 @@ const char **nm_utils_enum_get_values (GType type, gint from, gint to);
 NM_AVAILABLE_IN_1_6
 guint nm_utils_version (void);
 
+NM_AVAILABLE_IN_1_8
+GHashTable * nm_utils_parse_variant_attributes (const char *string,
+                                                char attr_separator,
+                                                char key_value_separator,
+                                                gboolean ignore_unknown,
+                                                const NMVariantAttributeSpec *const *spec,
+                                                GError **error);
+
+NM_AVAILABLE_IN_1_8
+char * nm_utils_format_variant_attributes (GHashTable *attributes,
+                                           char attr_separator,
+                                           char key_value_separator);
+
+/*****************************************************************************/
+
+NM_AVAILABLE_IN_1_12
+NMTCQdisc *nm_utils_tc_qdisc_from_str      (const char *str, GError **error);
+NM_AVAILABLE_IN_1_12
+char *nm_utils_tc_qdisc_to_str             (NMTCQdisc *qdisc, GError **error);
+
+NM_AVAILABLE_IN_1_12
+NMTCAction *nm_utils_tc_action_from_str    (const char *str, GError **error);
+NM_AVAILABLE_IN_1_12
+char *nm_utils_tc_action_to_str            (NMTCAction *action, GError **error);
+
+NM_AVAILABLE_IN_1_12
+NMTCTfilter *nm_utils_tc_tfilter_from_str  (const char *str, GError **error);
+NM_AVAILABLE_IN_1_12
+char *nm_utils_tc_tfilter_to_str           (NMTCTfilter *tfilter, GError **error);
+
 G_END_DECLS
 
 #endif /* __NM_UTILS_H__ */
