@@ -20,18 +20,15 @@
  * Copyright (C) 2007 - 2008 Red Hat, Inc.
  */
 
-#include "config.h"
+#include "nm-default.h"
 
 #include <stdlib.h>
 #include <signal.h>
 #include <string.h>
-
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 
-
-#include "nm-default.h"
 #include "nm-client.h"
 #include "nm-device.h"
 #include "nm-device-ethernet.h"
@@ -50,12 +47,12 @@ test_wireless_enabled (NMClient *client)
 	wireless = nm_client_wireless_get_enabled (client);
 	g_print ("Wireless enabled? %s\n", wireless ? "yes" : "no");
 
-//	nm_client_wireless_set_enabled (client, !wireless);
+// nm_client_wireless_set_enabled (client, !wireless);
 
 	wireless = nm_client_wireless_hardware_get_enabled (client);
 	g_print ("Wireless HW enabled? %s\n", wireless ? "yes" : "no");
 
-//	nm_client_wireless_set_enabled (client, !wireless);
+// nm_client_wireless_set_enabled (client, !wireless);
 
 	return TRUE;
 }
@@ -403,8 +400,6 @@ main (int argc, char *argv[])
 {
 	NMClient *client;
 
-	nm_g_type_init ();
-
 	client = nm_client_new ();
 	if (!client) {
 		exit (1);
