@@ -27,6 +27,7 @@
 #include "nm-secret-agent.h"
 #include "nm-glib-enum-types.h"
 #include "nm-dbus-helpers-private.h"
+#include "nm-setting-private.h"
 
 static void impl_secret_agent_get_secrets (NMSecretAgent *self,
                                            GHashTable *connection_hash,
@@ -98,7 +99,7 @@ enum {
 static guint signals[LAST_SIGNAL] = { 0 };
 
 
-/********************************************************************/
+/*****************************************************************************/
 
 GQuark
 nm_secret_agent_error_quark (void)
@@ -110,7 +111,7 @@ nm_secret_agent_error_quark (void)
 	return ret;
 }
 
-/*************************************************************/
+/*****************************************************************************/
 
 static const char *
 get_nm_owner (NMSecretAgent *self)
@@ -302,7 +303,8 @@ verify_request (NMSecretAgent *self,
                 GError **error)
 {
 	NMConnection *connection = NULL;
-	GError *local = NULL;
+
+	g_return_val_if_fail (!connection_hash || out_connection, FALSE);
 
 	if (!verify_sender (self, context, error))
 		return FALSE;
@@ -321,22 +323,11 @@ verify_request (NMSecretAgent *self,
 	}
 
 	/* Make sure the given connection is valid */
-	g_assert (out_connection);
-	connection = nm_connection_new_from_hash (connection_hash, &local);
-	if (connection) {
-		nm_connection_set_path (connection, connection_path);
-		*out_connection = connection;
-	} else {
-		g_set_error (error,
-		             NM_SECRET_AGENT_ERROR,
-		             NM_SECRET_AGENT_ERROR_INVALID_CONNECTION,
-		             "Invalid connection: (%d) %s",
-		             local ? local->code : -1,
-		             (local && local->message) ? local->message : "(unknown)");
-		g_clear_error (&local);
-	}
+	connection = _nm_connection_new_from_hash (connection_hash);
+	nm_connection_set_path (connection, connection_path);
+	*out_connection = connection;
 
-	return !!connection;
+	return TRUE;
 }
 
 static void
@@ -521,7 +512,7 @@ impl_secret_agent_delete_secrets (NMSecretAgent *self,
 	g_object_unref (connection);
 }
 
-/**************************************************************/
+/*****************************************************************************/
 
 static void
 reg_result (NMSecretAgent *self, GError *error)
@@ -701,10 +692,10 @@ auto_register_cb (gpointer user_data)
 	return FALSE;
 }
 
-/**************************************************************/
+/*****************************************************************************/
 
 /**
- * nm_secret_agent_get_secrets:
+ * nm_secret_agent_get_secrets: (virtual get_secrets):
  * @self: a #NMSecretAgent
  * @connection: the #NMConnection for which we're asked secrets
  * @setting_name: the name of the secret setting
@@ -718,8 +709,6 @@ auto_register_cb (gpointer user_data)
  * agent should use when performing the request, for example returning only
  * existing secrets without user interaction, or requesting entirely new
  * secrets from the user.
- *
- * Virtual: get_secrets
  */
 void
 nm_secret_agent_get_secrets (NMSecretAgent *self,
@@ -748,7 +737,7 @@ nm_secret_agent_get_secrets (NMSecretAgent *self,
 }
 
 /**
- * nm_secret_agent_save_secrets:
+ * nm_secret_agent_save_secrets: (virtual save_secrets):
  * @self: a #NMSecretAgent
  * @connection: a #NMConnection
  * @callback: (scope async): a callback, to be invoked when the operation is done
@@ -756,8 +745,6 @@ nm_secret_agent_get_secrets (NMSecretAgent *self,
  *
  * Asyncronously ensure that all secrets inside @connection
  * are stored to disk.
- *
- * Virtual: save_secrets
  */
 void
 nm_secret_agent_save_secrets (NMSecretAgent *self,
@@ -777,7 +764,7 @@ nm_secret_agent_save_secrets (NMSecretAgent *self,
 }
 
 /**
- * nm_secret_agent_delete_secrets:
+ * nm_secret_agent_delete_secrets: (virtual delete_secrets):
  * @self: a #NMSecretAgent
  * @connection: a #NMConnection
  * @callback: (scope async): a callback, to be invoked when the operation is done
@@ -785,8 +772,6 @@ nm_secret_agent_save_secrets (NMSecretAgent *self,
  *
  * Asynchronously ask the agent to delete all saved secrets belonging to
  * @connection.
- *
- * Virtual: delete_secrets
  */
 void
 nm_secret_agent_delete_secrets (NMSecretAgent *self,
@@ -805,7 +790,7 @@ nm_secret_agent_delete_secrets (NMSecretAgent *self,
 	                                                  user_data);
 }
 
-/**************************************************************/
+/*****************************************************************************/
 
 static gboolean
 validate_identifier (const char *identifier)
