@@ -408,40 +408,53 @@ NM_AVAILABLE_IN_1_12
 const GPtrArray *nm_client_get_checkpoints (NMClient *client);
 
 NM_AVAILABLE_IN_1_12
-void nm_client_checkpoint_create_async (NMClient *client,
-                                        const GPtrArray *devices,
-                                        guint32 rollback_timeout,
-                                        NMCheckpointCreateFlags flags,
-                                        GCancellable *cancellable,
-                                        GAsyncReadyCallback callback,
-                                        gpointer user_data);
+void nm_client_checkpoint_create (NMClient *client,
+                                  const GPtrArray *devices,
+                                  guint32 rollback_timeout,
+                                  NMCheckpointCreateFlags flags,
+                                  GCancellable *cancellable,
+                                  GAsyncReadyCallback callback,
+                                  gpointer user_data);
 NM_AVAILABLE_IN_1_12
 NMCheckpoint *nm_client_checkpoint_create_finish (NMClient *client,
                                                   GAsyncResult *result,
                                                   GError **error);
 
 NM_AVAILABLE_IN_1_12
-void nm_client_checkpoint_destroy_async (NMClient *client,
-                                         NMCheckpoint *checkpoint,
-                                         GCancellable *cancellable,
-                                         GAsyncReadyCallback callback,
-                                         gpointer user_data);
+void nm_client_checkpoint_destroy (NMClient *client,
+                                   const char *checkpoint_path,
+                                   GCancellable *cancellable,
+                                   GAsyncReadyCallback callback,
+                                   gpointer user_data);
 NM_AVAILABLE_IN_1_12
 gboolean nm_client_checkpoint_destroy_finish (NMClient *client,
                                               GAsyncResult *result,
                                               GError **error);
 
 NM_AVAILABLE_IN_1_12
-void nm_client_checkpoint_rollback_async (NMClient *client,
-                                          NMCheckpoint *checkpoint,
-                                          GCancellable *cancellable,
-                                          GAsyncReadyCallback callback,
-                                          gpointer user_data);
+void nm_client_checkpoint_rollback (NMClient *client,
+                                    const char *checkpoint_path,
+                                    GCancellable *cancellable,
+                                    GAsyncReadyCallback callback,
+                                    gpointer user_data);
 NM_AVAILABLE_IN_1_12
 GHashTable *nm_client_checkpoint_rollback_finish (NMClient *client,
                                                   GAsyncResult *result,
                                                   GError **error);
 
+NM_AVAILABLE_IN_1_12
+void nm_client_checkpoint_adjust_rollback_timeout (NMClient *client,
+                                                   const char *checkpoint_path,
+                                                   guint32 add_timeout,
+                                                   GCancellable *cancellable,
+                                                   GAsyncReadyCallback callback,
+                                                   gpointer user_data);
+
+NM_AVAILABLE_IN_1_12
+gboolean nm_client_checkpoint_adjust_rollback_timeout_finish (NMClient *client,
+                                                              GAsyncResult *result,
+                                                              GError **error);
+
 G_END_DECLS
 
 #endif /* __NM_CLIENT_H__ */
