@@ -407,6 +407,8 @@ _device_wifi_request_scan_async (NMDeviceWifi *device,
 
 	simple = g_simple_async_result_new (G_OBJECT (device), callback, user_data,
 	                                    nm_device_wifi_request_scan_async);
+	if (cancellable)
+		g_simple_async_result_set_check_cancellable (simple, cancellable);
 
 	/* If a scan is in progress, just return */
 	if (priv->scan_info) {
@@ -885,11 +887,9 @@ nm_device_wifi_class_init (NMDeviceWifiClass *wifi_class)
 		                     G_PARAM_STATIC_STRINGS));
 
 	/**
-	 * NMDeviceWifi:access-points:
+	 * NMDeviceWifi:access-points: (type GPtrArray(NMAccessPoint))
 	 *
 	 * List of all Wi-Fi access points the device can see.
-	 *
-	 * Element-type: NMAccessPoint
 	 **/
 	g_object_class_install_property
 		(object_class, PROP_ACCESS_POINTS,
