@@ -304,7 +304,7 @@ nm_ip_config_class_init (NMIPConfigClass *config_class)
 	                         G_PARAM_STATIC_STRINGS));
 
 	/**
-	 * NMIPConfig:routes:
+	 * NMIPConfig:routes: (type GPtrArray(NMIPRoute))
 	 *
 	 * A #GPtrArray containing the routes (#NMIPRoute) of the configuration.
 	 **/
@@ -395,7 +395,7 @@ nm_ip_config_get_gateway (NMIPConfig *config)
 {
 	g_return_val_if_fail (NM_IS_IP_CONFIG (config), NULL);
 
-	return NM_IP_CONFIG_GET_PRIVATE (config)->gateway;
+	return nm_str_not_empty (NM_IP_CONFIG_GET_PRIVATE (config)->gateway);
 }
 
 /**
