@@ -42,6 +42,7 @@ G_BEGIN_DECLS
 #define NM_REMOTE_CONNECTION_DBUS_CONNECTION "dbus-connection"
 #define NM_REMOTE_CONNECTION_PATH            "path"
 #define NM_REMOTE_CONNECTION_UNSAVED         "unsaved"
+#define NM_REMOTE_CONNECTION_FLAGS           "flags"
 #define NM_REMOTE_CONNECTION_VISIBLE         "visible"
 
 /**
@@ -123,6 +124,9 @@ GVariant *nm_remote_connection_get_secrets_finish (NMRemoteConnection *connectio
 
 gboolean nm_remote_connection_get_unsaved (NMRemoteConnection *connection);
 
+NM_AVAILABLE_IN_1_12
+NMSettingsConnectionFlags nm_remote_connection_get_flags (NMRemoteConnection *connection);
+
 gboolean nm_remote_connection_get_visible (NMRemoteConnection *connection);
 
 G_END_DECLS
