@@ -349,10 +349,9 @@ connection_activated_none_cb (GObject *c,
                               gpointer user_data)
 {
 	TestSecretAgentData *sadata = user_data;
-	NMActiveConnection *ac;
 	gs_free_error GError *error = NULL;
 
-	ac = nm_client_activate_connection_finish (sadata->client, result, &error);
+	nm_client_activate_connection_finish (sadata->client, result, &error);
 	g_assert_error (error, NM_AGENT_MANAGER_ERROR, NM_AGENT_MANAGER_ERROR_NO_SECRETS);
 
 	g_main_loop_quit (sadata->loop);
@@ -599,6 +598,12 @@ test_secret_agent_auto_register (void)
 	g_assert_no_error (error);
 	g_assert (nm_secret_agent_old_get_registered (agent));
 
+	/* The GLib ObjectManager doesn't like when we drop the service
+	 * in between it sees the service disappear and the call to
+	 * GetManagedObjects. Give it a chance to do its business.
+	 * Arguably a bug. */
+	g_main_context_iteration (NULL, FALSE);
+
 	/* Shut down test service */
 	nmtstc_service_cleanup (sinfo);
 	g_main_loop_run (loop);
@@ -609,6 +614,9 @@ test_secret_agent_auto_register (void)
 	g_main_loop_run (loop);
 	g_assert (nm_secret_agent_old_get_registered (agent));
 
+	/* Let ObjectManager initialize (see above). */
+	g_main_context_iteration (NULL, FALSE);
+
 	/* Shut down test service again */
 	nmtstc_service_cleanup (sinfo);
 	g_main_loop_run (loop);
