@@ -24,8 +24,8 @@
 
 #include "nm-core-internal.h"
 
-#include "nms-ifupdown-interface-parser.h"
-#include "nms-ifupdown-parser.h"
+#include "settings/plugins/ifupdown/nms-ifupdown-interface-parser.h"
+#include "settings/plugins/ifupdown/nms-ifupdown-parser.h"
 
 #include "nm-test-utils-core.h"
 
@@ -168,7 +168,7 @@ dump_blocks (void)
 	for (n = ifparser_getfirst (); n != NULL; n = n->next) {
 		if_data *m;
 
-		// each block start with its type & name 
+		// each block start with its type & name
 		// (single quotes used to show typ & name baoundaries)
 		g_print("'%s' '%s'\n", n->type, n->name);
 
