@@ -7,10 +7,10 @@ die() {
 
 _is_true() {
     case "$1" in
-        y|Y|yes|YES|1)
+        y|Y|yes|YES|1|true|TRUE)
             return 0
             ;;
-        n|N|no|NO|0)
+        n|N|no|NO|0|false|FALSE)
             return 1
             ;;
         *)
@@ -27,26 +27,53 @@ SCRIPT_PATH="${SCRIPT_PATH:-$(readlink -f "$(dirname "$0")")}"
 
 VALGRIND_ERROR=37
 
-
-if [[ -z "${NMTST_USE_VALGRIND+x}" ]]; then
-    if [[ -z "${NMTST_NO_VALGRIND+x}" ]]; then
-        NMTST_USE_VALGRIND=0
-    elif [[ -z "${NMTST_NO_VALGRIND}" ]]; then
-        NMTST_USE_VALGRIND=1
-    else
-        NMTST_USE_VALGRIND=0
-    fi
-fi
-
 if [ "$1" == "--called-from-make" ]; then
     shift
-    NMTST_LIBTOOL=($1 --mode=execute); shift
-    NMTST_VALGRIND="$1"; shift
-    if [[ "$NMTST_VALGRIND" == no ]]; then
-        NMTST_USE_VALGRIND=0
-        NMTST_VALGRIND=
+    CALLED_FROM_MAKE=1
+else
+    CALLED_FROM_MAKE=0
+fi
+
+if [ "$CALLED_FROM_MAKE" == 1 ]; then
+    if [ -n "$1" ]; then
+        NMTST_LIBTOOL=($1 --mode=execute);
+    else
+        NMTST_LIBTOOL=()
     fi
-    SUPPRESSIONS="$1"; shift
+    shift
+    NMTST_VALGRIND_ARG="$1"; shift
+    if [[ "$NMTST_VALGRIND_ARG" == no ]]; then
+        NMTST_VALGRIND_ARG=
+    fi
+
+    if [[ -z "${NMTST_VALGRIND}" ]]; then
+        # the valgrind path can be specified via $NMTST_VALGRIND.
+        # Otherwise, it can be determined by the configure scripts.
+        # Otherwise, it is found in the current $PATH (below).
+        if [[ "$NMTST_VALGRIND_ARG" != "" ]]; then
+            NMTST_VALGRIND="${NMTST_VALGRIND_ARG}"
+        fi
+    fi
+    if [[ -z "${NMTST_USE_VALGRIND+x}" ]]; then
+        # whether to use valgrind can be specified via $NMTST_USE_VALGRIND.
+        # Otherwise, it depends on the configure option.
+        if [ "$NMTST_VALGRIND_ARG" == "" ]; then
+            NMTST_USE_VALGRIND=0
+        else
+            NMTST_USE_VALGRIND=1
+        fi
+    fi
+
+    NMTST_SUPPRESSIONS_ARGS="$1"; shift
+    if [[ -z "${NMTST_SUPPRESSIONS+x}" ]]; then
+        if [[ "$NMTST_SUPPRESSIONS_ARGS" == "" ]]; then
+            NMTST_SUPPRESSIONS="$SCRIPT_PATH/../valgrind.suppressions"
+        else
+            NMTST_SUPPRESSIONS="${NMTST_SUPPRESSIONS_ARGS}"
+        fi
+    fi
+
+
     if [ "$1" = "--launch-dbus" ]; then
         NMTST_LAUNCH_DBUS=1
         shift
@@ -60,6 +87,11 @@ if [ "$1" == "--called-from-make" ]; then
     NMTST_MAKE_FIRST=0
 
 else
+    if [[ -z "${NMTST_USE_VALGRIND+x}" ]]; then
+        # by default, disable valgrind checks.
+        NMTST_USE_VALGRIND=0
+    fi
+
     if [ -n "${NMTST_LIBTOOL-:x}" ]; then
         NMTST_LIBTOOL=(sh "$SCRIPT_PATH/../libtool" --mode=execute)
     elif [ -n "${NMTST_LIBTOOL-x}" ]; then
@@ -67,8 +99,9 @@ else
     else
         NMTST_LIBTOOL=($NMTST_LIBTOOL --mode=execute)
     fi
-    for a in "$@"; do
-        case "$a" in
+    unset TEST
+    while test $# -gt 0; do
+        case "$1" in
         "--launch-dbus")
             NMTST_LAUNCH_DBUS=1
             shift
@@ -93,6 +126,15 @@ else
             NMTST_USE_VALGRIND=0
             shift;
             ;;
+        "-d")
+            NMTST_SET_DEBUG=1
+            shift;
+            ;;
+        "--test"|-t)
+            shift
+            TEST="$1"
+            shift
+            ;;
         "--")
             shift
             break
@@ -104,13 +146,27 @@ else
     done
     # we support calling the script directly. In this case,
     # only pass the path to the test to run.
-    TEST="$1"; shift
-    if [ "$SUPPRESSIONS" == "" ]; then
-        SUPPRESSIONS="$SCRIPT_PATH/../valgrind.suppressions"
+    if test -z "${TEST+x}"; then
+        TEST="$1"; shift
+    fi
+    if [[ -z "${NMTST_SUPPRESSIONS+x}" ]]; then
+        NMTST_SUPPRESSIONS="$SCRIPT_PATH/../valgrind.suppressions"
     fi
 
 fi
 
+if [ "$NMTST_SET_DEBUG" == 1 -a -z "${NMTST_DEBUG+x}" ]; then
+    export NMTST_DEBUG=d
+fi
+
+if _is_true "$NMTST_MAKE_FIRST" 0; then
+    git_dir="$(readlink -f "$(git rev-parse --show-toplevel)")"
+    rel_path="$(realpath --relative-to="$git_dir" -m "$TEST" 2>/dev/null)" || die "cannot resolve test-name \"$TEST\". Did you call the script properly?"
+    cd "$git_dir"
+    make -j5 "$rel_path" || die "make of $TEST failed ($git_dir / $rel_path)"
+    cd - 1>/dev/null
+fi
+
 [ -x "$TEST" ] || die "Test \"$TEST\" does not exist"
 TEST_PATH="$(readlink -f "$(dirname "$TEST")")"
 TEST_NAME="${TEST##*/}"
@@ -124,14 +180,6 @@ if [ -z "${NMTST_LAUNCH_DBUS}" ]; then
     fi
 fi
 
-if [[ "$NMTST_MAKE_FIRST" == 1 ]]; then
-    git_dir="$(readlink -f "$(git rev-parse --show-toplevel)")"
-    rel_path="${TEST_PATH/#$(printf '%s/' "$git_dir")}/$TEST_NAME"
-    cd "$git_dir"
-    make -j5 "$rel_path" || die "make of $TEST failed ($git_dir / $rel_path)"
-    cd - 1>/dev/null
-fi
-
 # if the user wishes, change first into the directory of the test
 if _is_true "$NMTST_CHANGE_DIRECTORY" 0; then
     cd "$TEST_PATH"
@@ -163,6 +211,12 @@ else
     test -e "${NMTST_VALGRIND}" || die "cannot find valgrind binary from NMTST_VALGRIND=\"${NMTST_VALGRIND}\""
 fi
 
+if [[ "${NMTST_SUPPRESSIONS}" != "" ]]; then
+    NMTST_SUPPRESSIONS=("--suppressions=$NMTST_SUPPRESSIONS")
+else
+    NMTST_SUPPRESSIONS=()
+fi
+
 LOGFILE="${TEST}.valgrind-log"
 
 export G_SLICE=always-malloc
@@ -174,7 +228,7 @@ export G_DEBUG=gc-friendly
     --error-exitcode=$VALGRIND_ERROR \
     --leak-check=full \
     --gen-suppressions=all \
-    --suppressions="$SUPPRESSIONS" \
+    "${NMTST_SUPPRESSIONS[@]}" \
     --num-callers=100 \
     --log-file="$LOGFILE" \
     "$TEST" \
